"""Angular module commands"""

from devops_toolset.core.ValueDictsBase import ValueDictsBase
from devops_toolset.core.app import App

app: App = App()


class Commands(ValueDictsBase):
    """ Commands for the Angular module."""

    # Add your Angular commands dictionaries here
    _commands = {}
